package com.lindosoft.android.guide.controller;

import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.PowerManager;
import android.provider.Settings;
import com.lindosoft.android.guide.model.MainModel;

/** Start and stop location listener. */
public abstract class AbstractLocationListActivity extends AbstractListActivity implements LocationListener {
  protected Location location = null;
  protected LocationManager locationManager = null;
  protected boolean notify = false;
  protected PowerManager.WakeLock wl = null;

  @Override
  public synchronized void onLocationChanged(Location location) {
    this.location = location;
    handleLocation();
  }

  @Override
  public void onProviderDisabled(String provider) {
  }

  @Override
  public void onProviderEnabled(String provider) {
  }

  @Override
  public void onStatusChanged(String provider, int status, Bundle extras) {
  }

  /** Handle the latest location change. */
  protected abstract void handleLocation();

  /** Audible notification that gps became available. */
  protected void notifyGpsActive() {
    if (notify) {
      playNotification();
    }
    notify = false;
  }

  /** Play notification. */
  protected void playNotification() {
    Uri notification = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
    Ringtone r = RingtoneManager.getRingtone(getApplicationContext(), notification);
    r.play();
  }

  /** Show specified location on map. */
  protected void showMap(Location locationTemp) {
    if (null == locationTemp) {
      return;
    }
    String geo = "geo:0,0?z=17";
    geo += "&q=" + locationTemp.getLatitude() + "," + locationTemp.getLongitude();
    Uri uri = Uri.parse(geo);
    Intent intent = new Intent(Intent.ACTION_VIEW, uri);
    startActivity(intent);
  }

  /** Register this as listener for GPS location provider. */
  protected void startLocation() {
    if (null != locationManager) {
      return;
    }
    PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
    wl = pm.newWakeLock(PowerManager.SCREEN_DIM_WAKE_LOCK | PowerManager.ON_AFTER_RELEASE, "lindoguide");
    wl.acquire();
    location = null;
    notify = true;
    locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
    boolean gpsEnabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
    if (!gpsEnabled) {
      Intent settingsIntent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
      startActivity(settingsIntent);
    }
    locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, MainModel.intervalRecord, MainModel.radiusRecord, this);
  }

  /** Unregister this as listener for GPS location provider. */
  protected void stopLocation() {
    if (null != locationManager) {
      locationManager.removeUpdates(this);
      locationManager = null;
      location = null;
      wl.release();
    }
  }
}