package com.lindosoft.android.guide.controller;

import android.app.Activity;
import android.content.Context;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;
import com.lindosoft.android.guide.model.Point;
import com.lindosoft.android.guide.model.SightModel;

/** Play single point. */
public abstract class AbstractPlayActivity extends Activity implements AudioManager.OnAudioFocusChangeListener,
    MediaPlayer.OnCompletionListener {
  private static MediaPlayer playerPrevious = null;
  private MediaPlayer player = null;
  private static int streamVolume = -1;
  private static int streamVolumeOther = -1;

  @Override
  public void onBackPressed() {
    stopPlayer();
    super.onBackPressed();
  }

  @Override
  public void onAudioFocusChange(int focusChange) {
  }

  @Override
  public void onCompletion(MediaPlayer mediaPlayer) {
    AudioManager audioManager = (AudioManager) getSystemService(Context.AUDIO_SERVICE);
    streamVolume = audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
    stopPlayer();
    finish();
  }

  /** Handle stop button. */
  public void stop(View view) {
    stopPlayer();
    finish();
  }


  /** Specify the point data source. */
  protected abstract String getDataSource();

  /** Specify the point name. */
  protected abstract String getName();

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.play);
  }

  @Override
  protected void onDestroy() {
    if (null != player) {
      player.release();
    }
    player = null;
    super.onDestroy();
  }

  @Override
  protected void onResume() {
    super.onResume();
    TextView pointName = (TextView) findViewById(R.id.text_point_name);
    pointName.setText(getName());
    playPoint();
  }


  /** Play point audio. */
  private void playPoint() {
    AudioManager audioManager = (AudioManager) getSystemService(Context.AUDIO_SERVICE);
    // stop previous player
    try {
      playerPrevious.stop();
      streamVolume = audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
    } catch (Exception e) {
      streamVolumeOther = audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
    }
    if (-1 == streamVolume) {
      streamVolume = streamVolumeOther;
    }
    // start this player
    try {
      int resourceId = getResources().getIdentifier(getName(), "raw", getClass().getPackage().getName());
      player = MediaPlayer.create(this, resourceId);
      player.setOnCompletionListener(this);
      player.setScreenOnWhilePlaying(true);
    } catch (Exception ignore) {
      try {
        player = new MediaPlayer();
        player.setOnCompletionListener(this);
        player.setScreenOnWhilePlaying(true);
        player.setDataSource(getDataSource());
        player.prepare();
      } catch (Exception ex) {
        stopPlayer();
        Toast.makeText(this, "playPoint(): " + ex, Toast.LENGTH_LONG).show();
        finish();
      }
    }
    try {
      audioManager.requestAudioFocus(this, AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN_TRANSIENT_MAY_DUCK);
      player.start();
      playerPrevious = player;
      if (-1 != streamVolume) {
        audioManager.setStreamVolume(AudioManager.STREAM_MUSIC, streamVolume, 0);
      }
    } catch (Exception ex) {
      stopPlayer();
      Toast.makeText(this, "playPoint(): " + ex, Toast.LENGTH_LONG).show();
      finish();
    }
  }

  /** Stop player on abandon audio focus. */
  private void stopPlayer() {
    try {
      AudioManager audioManager = (AudioManager) getSystemService(Context.AUDIO_SERVICE);
      audioManager.abandonAudioFocus(this);
      if (player.isPlaying()) {
        streamVolume = audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
        player.stop();
      }
      if (-1 != streamVolumeOther) {
        audioManager.setStreamVolume(AudioManager.STREAM_MUSIC, streamVolumeOther, 0);
      }
    } catch (Exception ex) {
    }
  }
}