package com.lindosoft.android.guide.controller;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.location.Location;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import com.lindosoft.android.guide.model.MainModel;
import com.lindosoft.android.guide.model.Point;
import com.lindosoft.android.guide.model.SightModel;

/** Show list of sightpoints for current sight. */
public class SightpointsActivity extends AbstractLocationListActivity {
  @Override
  public void onBackPressed() {
    stopLocation();
    super.onBackPressed();
  }

  @Override
  public boolean onContextItemSelected(MenuItem item) {
    AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
    int menuItemIndex = item.getItemId();
    ArrayAdapter<Point> adapter = (ArrayAdapter<Point>) getListAdapter();
    Point sightpoint = adapter.getItem(info.position);
    switch(menuItemIndex) {
      case 0:
        recordSightpoint(sightpoint);
        break;
      case 1:
        adapter.remove(sightpoint);
        SightModel.removeSightpoint(sightpoint);
        break;
      case 2:
        SightModel.sightpoint = sightpoint;
        showMap(sightpoint.location);
        break;
    }
    return true;
  }

  @Override
  public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
    if (v != getListView()) {
      return;
    }
    AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) menuInfo;
    ArrayAdapter<Point> adapter = (ArrayAdapter<Point>) getListAdapter();
    Point sightpoint = adapter.getItem(info.position);
    SightModel.sightpoint = sightpoint;
    scroll(info.position);
    String[] menuItems = getResources().getStringArray(R.array.sightpoints_context_menu);
    for (int i = 0; i<menuItems.length; i++) {
      if (2 == i && null == sightpoint.location) {
        continue;
      }
      menu.add(Menu.NONE, i, i, menuItems[i]);
    }
  }

  /** Handle create sightpoint button. */
  public void onCreateSightpoint(View view) {
    AlertDialog.Builder alert = new AlertDialog.Builder(this);
    alert.setIcon(R.drawable.file_new);
    alert.setMessage(R.string.alert_message_sightpoint_new);
    alert.setTitle(R.string.alert_title_sightpoint_new);
    LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    final EditText editSightpointName = (EditText) inflater.inflate(R.layout.sightpoint_new, null);
    alert.setView(editSightpointName);
    alert.setPositiveButton(R.string.alert_ok_sightpoint_new, new DialogInterface.OnClickListener() {
      public void onClick(DialogInterface dialog, int whichButton) {
        String name = editSightpointName.getText().toString();
        Point sightpoint = SightModel.createSightpoint(location, name);
        recordSightpoint(sightpoint);
      }
    });
    alert.show();
  }


  @Override
  protected void handleLocation() {
    Button button = (Button) findViewById(R.id.button_create_sightpoint);
    button.setCompoundDrawablesWithIntrinsicBounds(R.drawable.file_new_gps, 0, 0, 0);
    notifyGpsActive();
    Point bestSightpoint = getBestSightpoint(location);
    if (null != bestSightpoint && !bestSightpoint.equals(SightModel.sightpoint)) {
      playSightpoint(bestSightpoint);
    }
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.sightpoints);
    ArrayAdapter<Point> adapter = new ArrayAdapter<Point>(this, R.layout.sightpoints_item, R.id.label) {
      public View getView(int position, View convertView, ViewGroup parent) {
        View view = super.getView(position, convertView, parent);
        if (null != SightModel.sightpoint && position == getPosition(SightModel.sightpoint)) {
          view.setBackgroundColor(Color.DKGRAY);
        } else {
          view.setBackgroundColor(Color.BLACK);
        }
        return view;
      }
		};
    setListAdapter(adapter);
  }
  
  @Override
  protected void onDestroy() {
    stopLocation();
    super.onDestroy();
  }

  @Override
  protected void onListItemClick(ListView l, View v, int position, long id) {
    playSightpoint((Point) getListAdapter().getItem(position));
  }

  @Override
  protected void onResume() {
    super.onResume();
    SightModel.readSightpoints();
    ArrayAdapter<Point> adapter = (ArrayAdapter<Point>) getListAdapter();
    adapter.clear();
    for (Point point : SightModel.sightpoints) {
      adapter.add(point);
    }
    TextView sightName = (TextView) findViewById(R.id.text_sight_name);
    sightName.setText(SightModel.sight.getName());
    registerForContextMenu(getListView());
    playSightpointHeader();
    startLocation();
    Button button = (Button) findViewById(R.id.button_create_sightpoint);
    button.setCompoundDrawablesWithIntrinsicBounds(R.drawable.file_new, 0, 0, 0);
  }

  
  /** Get the sightpoint closest to the specified location within play radius. */
  private Point getBestSightpoint(Location locationTemp) {
    Point bestSightpoint = null;
    if (null != SightModel.sight) {
      float distance = MainModel.radiusPlay;
      for (Point sightpoint : SightModel.sightpoints) {
        if (null == sightpoint.location) {
          continue;
        }
        float distanceTo = locationTemp.distanceTo(sightpoint.location);
        if (distanceTo <= distance) {
          distance = distanceTo;
          bestSightpoint = sightpoint;
        }
      }
    }
    return bestSightpoint;
  }

  /** Play the first sightpoint without location. */
  private void playSightpointHeader() {
    if (null == SightModel.sight || null != SightModel.sightpoint) {
      return;
    }
    for (Point sightpoint : SightModel.sightpoints) {
      if (null == sightpoint.location) {
        playSightpoint(sightpoint);
        break;
      }
    }
  }     
   
  /** Play specified sightpint. */
  private void playSightpoint(Point sightpoint) {
    SightModel.sightpoint = sightpoint;
    Intent intent = new Intent(this, SightpointPlayActivity.class);
    startActivity(intent);
  }

  /** Record specified sightpoint. */
  private void recordSightpoint(Point sightpoint) {
    stopLocation();
    SightModel.sightpoint = sightpoint;
    Intent intent = new Intent(this, SightpointRecordActivity.class);
    startActivity(intent);
  }
}