package com.lindosoft.android.guide.controller;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import com.lindosoft.android.guide.model.MainModel;
import com.lindosoft.android.guide.model.SightModel;
import java.io.File;
import java.util.Collections;

/** Show list of sights. */
public class SightsActivity extends AbstractListActivity {
  @Override
  public boolean onContextItemSelected(MenuItem item) {
    AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
    ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
    File sight = adapter.getItem(info.position);
    int menuItemIndex = item.getItemId();
    if (0 == menuItemIndex) {
      removeSight(sight);
    } else if (1 == menuItemIndex) {
      renameSight(sight);
    }
    return true;
  }
  
  @Override
  public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
    if (v != getListView()) {
      return;
    }
    AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) menuInfo;
    ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
    File sight = adapter.getItem(info.position);
    SightModel.sight = sight;
    scroll(info.position);
    String[] menuItems = getResources().getStringArray(R.array.sights_context_menu);
    for (int i = 0; i<menuItems.length; i++) {
      menu.add(Menu.NONE, i, i, menuItems[i]);
    }
  }

  /** Handle create sight button. */
  public void onCreateSight(View view) {
    AlertDialog.Builder alert = new AlertDialog.Builder(this);
    alert.setIcon(R.drawable.folder_new);
    alert.setMessage(R.string.alert_message_sight_new);
    alert.setTitle(R.string.alert_title_sight_new);
    LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    final EditText editSightName = (EditText) inflater.inflate(R.layout.sight_new, null);
    alert.setView(editSightName);
    alert.setPositiveButton(R.string.alert_ok_sight_new, new DialogInterface.OnClickListener() {
      public void onClick(DialogInterface dialog, int whichButton) {
        String name = editSightName.getText().toString();
        File sight = SightModel.createSight(name);
        showSight(sight);
      }
    });
    alert.show();
  }


  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.sights);
    ArrayAdapter<File> adapter = new ArrayAdapter<File>(this, R.layout.sights_item, R.id.label) {
      public View getView(int position, View convertView, ViewGroup parent) {
        View view = super.getView(position, convertView, parent);
        TextView label = (TextView) view.findViewById(R.id.label);
        label.setText(SightModel.sights.get(position).getName());
        if (null != SightModel.sight && position == getPosition(SightModel.sight)) {
          view.setBackgroundColor(Color.DKGRAY);
        } else {
          view.setBackgroundColor(Color.BLACK);
        }
        return view;
      }
		};
    setListAdapter(adapter);
    registerForContextMenu(getListView());
  }
  
  @Override
  protected void onListItemClick(ListView l, View v, int position, long id) {
    showSight((File) getListAdapter().getItem(position));
  }

  @Override
  protected void onResume() {
    super.onResume();
    SightModel.readSights();
    ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
    adapter.clear();
    for (File file : SightModel.sights) {
      adapter.add(file);
    }
    scroll(adapter.getPosition(SightModel.sight));
  }

  /** Remove sight from sights and from filesystem. */
  private void removeSight(File sight) {
    ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
    adapter.remove(sight);
    SightModel.removeSight(sight);
  }

  /** Rename sight. */
  public void renameSight(final File sight) {
    AlertDialog.Builder alert = new AlertDialog.Builder(this);
    alert.setIcon(R.drawable.folder);
    alert.setTitle(R.string.alert_title_sight_rename);
    LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    final EditText editText = (EditText) inflater.inflate(R.layout.sight_rename, null);
    final String nameOld = sight.getName();
    editText.setText(nameOld);
    alert.setView(editText);
    alert.setPositiveButton(R.string.alert_ok_sight_rename, new DialogInterface.OnClickListener() {
      public void onClick(DialogInterface dialog, int whichButton) {
        String name = editText.getText().toString();
        if (0 == name.length() || name.equalsIgnoreCase(nameOld)) {
          return;
        }
        File sightNew = new File(sight.getParentFile(), name);
        if(sight.renameTo(sightNew)) {
          File sightpointHeader = new File(sightNew, nameOld + MainModel.audioExtension);
          if (sightpointHeader.isFile()) {
            File sightpointHeaderNew = new File(sightNew, name + MainModel.audioExtension);
            sightpointHeader.renameTo(sightpointHeaderNew);
          }
          SightModel.sights.remove(sight);
          SightModel.sights.add(sightNew);
          SightModel.sight = sightNew;
          Collections.sort(SightModel.sights);
          ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
          adapter.clear();
          for (File sight : SightModel.sights) {
            adapter.add(sight);
          }
          scroll(adapter.getPosition(SightModel.sight));
        }
      }
    });
    alert.show();
  }

  /** Show specified sight. */
  private void showSight(File sight) {
    SightModel.sight = sight;
    Intent intent = new Intent(this, SightpointsActivity.class);
    startActivity(intent);
  }
}