package com.lindosoft.android.guide.controller;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.location.Location;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import com.lindosoft.android.guide.model.MainModel;
import com.lindosoft.android.guide.model.Point;
import com.lindosoft.android.guide.model.SyncPlayer;
import com.lindosoft.android.guide.model.TourModel;

/** Show list of tourpoints for current tour. */
public class TourpointsActivity extends AbstractLocationListActivity {
  private SyncPlayer player = null;
  private long timestamp = 0L;

  @Override
  public void onBackPressed() {
    stopLocation();
    super.onBackPressed();
  }

  @Override
  public boolean onContextItemSelected(MenuItem item) {
    AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
    int menuItemIndex = item.getItemId();
    ArrayAdapter<Point> adapter = (ArrayAdapter<Point>) getListAdapter();
    Point tourpoint = adapter.getItem(info.position);
    switch(menuItemIndex) {
      case 0:
        TourModel.setNearest(location);
        scroll(TourModel.getTourpointPosition());
        timestamp = 0;        
        break;
      case 1:
        TourModel.tourpoint = tourpoint;
        scroll(info.position);
        timestamp = 0;
        break;
      case 2:
        recordTourpoint(tourpoint);
        break;
      case 3:
        adapter.remove(tourpoint);
        try {
          TourModel.removeTourpoint(tourpoint);
        } catch (Exception ex) {
          Toast.makeText(this, "removeTourpoint(): " + ex, Toast.LENGTH_LONG).show();
        }
        break;
      case 4:
        TourModel.tourpoint = tourpoint;
        showMap(tourpoint.location);
        break;
    }
    return true;
  }

  @Override
  public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
    if (v != getListView()) {
      return;
    }
    String[] menuItems = getResources().getStringArray(R.array.tourpoints_context_menu);
    for (int i = 0; i<menuItems.length; i++) {
      menu.add(Menu.NONE, i, i, menuItems[i]);
    }
  }

  /** Handle create tourpoint button. */
  public void onCreateTourpoint(View view) {
    if (null == location) {
      return;
    }
    stopLocation();
    Intent intent = new Intent(this, TourpointNewActivity.class);
    startActivity(intent);
  }


  @Override
  protected void handleLocation() {
    Button button = (Button) findViewById(R.id.button_create_tourpoint);
    button.setCompoundDrawablesWithIntrinsicBounds(R.drawable.file_new_gps, 0, 0, 0);
    if (0 == TourModel.tourpoints.size() || null == TourModel.tourpoint) {
      return;
    }
    if (location.distanceTo(TourModel.tourpoint.location) < MainModel.radiusPlay) {
      timestamp = System.currentTimeMillis();
      play(TourModel.tourpoint.name, TourModel.getTourpointAbsolutePath(), true);
      int position = TourModel.getTourpointPosition() + 1;
      TourModel.tourpoint = TourModel.tourpoints.size() == position ? null : TourModel.tourpoints.get(position);
      scroll(TourModel.getTourpointPosition());
      return;
    }
    if (System.currentTimeMillis() - timestamp > MainModel.intervalPlay) {
      // timeout reached, probably off tour, direct to current tourpoint
      timestamp = System.currentTimeMillis();
      Location locationTarget = TourModel.tourpoint.location;
      int direction = TourModel.getDirection(location, null, locationTarget);
      if (direction < 12) {
        play("b" + direction, null, true);
      }
      int distance = TourModel.getDistance(location, locationTarget);
      play("d" + distance, null, true);
    }
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.tourpoints);
    player = new SyncPlayer((AudioManager) getSystemService(Context.AUDIO_SERVICE), null, true);
    ArrayAdapter<Point> adapter = new ArrayAdapter<Point>(this, R.layout.tourpoints_item, R.id.label) {
      public View getView(int position, View convertView, ViewGroup parent) {
        View view = super.getView(position, convertView, parent);
        if (null != TourModel.tourpoint && position == getPosition(TourModel.tourpoint)) {
          view.setBackgroundColor(Color.DKGRAY);
        } else {
          view.setBackgroundColor(Color.BLACK);
        }
        return view;
      }
		};
    setListAdapter(adapter);
  }
  
  @Override
  protected void onDestroy() {
    stopLocation();
    super.onDestroy();
  }

  @Override
  protected void onListItemClick(ListView l, View v, int position, long id) {
    playTourpoint((Point) getListAdapter().getItem(position));
  }

  @Override
  protected void onResume() {
    super.onResume();
    try {
      TourModel.readTourpoints();
    } catch (Exception ex) {
      Toast.makeText(this, "readTourpoints(): " + ex, Toast.LENGTH_LONG).show();
    }
    ArrayAdapter<Point> adapter = (ArrayAdapter<Point>) getListAdapter();
    adapter.clear();
    for (Point point : TourModel.tourpoints) {
      adapter.add(point);
    }
    TextView tourName = (TextView) findViewById(R.id.text_tour_name);
    tourName.setText(TourModel.tour.getName());
    scroll(TourModel.getTourpointPosition());
    registerForContextMenu(getListView());
    startLocation();
    Button button = (Button) findViewById(R.id.button_create_tourpoint);
    button.setCompoundDrawablesWithIntrinsicBounds(R.drawable.file_new, 0, 0, 0);
  }


  /** Play audio resource. */
  private void play(String resource, String dataSource, boolean startNext) {
    MediaPlayer mediaPlayer = null;
    try {
      int resourceId = getResources().getIdentifier(resource, "raw", getClass().getPackage().getName());
      mediaPlayer = MediaPlayer.create(this, resourceId);
    } catch (Exception ignore) {
      if (null == dataSource) {
        Toast.makeText(this, "play(" + resource + "): " + ignore, Toast.LENGTH_LONG).show();
      } else {
        try {
          mediaPlayer = new MediaPlayer();
          mediaPlayer.setDataSource(dataSource);
          mediaPlayer.prepare();
        } catch (Exception ex) {
          mediaPlayer.release();
          Toast.makeText(this, "play(" + resource + "): " + ex, Toast.LENGTH_LONG).show();
        }
      }
    }
    SyncPlayer next = new SyncPlayer((AudioManager) getSystemService(Context.AUDIO_SERVICE), mediaPlayer, startNext);
    player = player.setNext(next);
  }
  
  /** Play specified tourpoint. */
  private void playTourpoint(Point tourpoint) {
    stopLocation();
    TourModel.tourpoint = tourpoint;
    Intent intent = new Intent(this, TourpointPlayActivity.class);
    startActivity(intent);
  }

  /** Record specified tourpoint. */
  private void recordTourpoint(Point tourpoint) {
    stopLocation();
    TourModel.tourpoint = tourpoint;
    Intent intent = new Intent(this, TourpointRecordActivity.class);
    startActivity(intent);
  }
}