package com.lindosoft.android.guide.controller;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import com.lindosoft.android.guide.model.TourModel;
import java.io.File;
import java.util.Collections;

/** Show list of tours. */
public class ToursActivity extends AbstractListActivity {
  @Override
  public boolean onContextItemSelected(MenuItem item) {
    AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
    ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
    File tour = adapter.getItem(info.position);
    int menuItemIndex = item.getItemId();
    if (0 == menuItemIndex) {
      removeTour(tour);
    } else if (1 == menuItemIndex) {
      renameTour(tour);
    }
    return true;
  }
  
  @Override
  public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
    if (v != getListView()) {
      return;
    }
    AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) menuInfo;
    ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
    File tour = adapter.getItem(info.position);
    TourModel.tour = tour;
    scroll(info.position);
    String[] menuItems = getResources().getStringArray(R.array.tours_context_menu);
    for (int i = 0; i<menuItems.length; i++) {
      menu.add(Menu.NONE, i, i, menuItems[i]);
    }
  }

  /** Handle create tour button. */
  public void onCreateTour(View view) {
    AlertDialog.Builder alert = new AlertDialog.Builder(this);
    alert.setIcon(R.drawable.folder_new);
    alert.setMessage(R.string.alert_message_tour_new);
    alert.setTitle(R.string.alert_title_tour_new);
    LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    final EditText editTourName = (EditText) inflater.inflate(R.layout.tour_new, null);
    alert.setView(editTourName);
    alert.setPositiveButton(R.string.alert_ok_tour_new, new DialogInterface.OnClickListener() {
      public void onClick(DialogInterface dialog, int whichButton) {
        String name = editTourName.getText().toString();
        try {
          File tour = TourModel.createTour(name);
          showTour(tour);
        } catch (Exception ex) {
          Toast.makeText(ToursActivity.this, "createTour(" + name + "): " + ex, Toast.LENGTH_LONG).show();
        }
      }
    });
    alert.show();
  }


  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.tours);
    ArrayAdapter<File> adapter = new ArrayAdapter<File>(this, R.layout.tours_item, R.id.label) {
      public View getView(int position, View convertView, ViewGroup parent) {
        View view = super.getView(position, convertView, parent);
        TextView label = (TextView) view.findViewById(R.id.label);
        label.setText(TourModel.tours.get(position).getName());
        if (null != TourModel.tour && position == getPosition(TourModel.tour)) {
          view.setBackgroundColor(Color.DKGRAY);
        } else {
          view.setBackgroundColor(Color.BLACK);
        }
        return view;
      }
		};
    setListAdapter(adapter);
    registerForContextMenu(getListView());
  }
  
  @Override
  protected void onListItemClick(ListView l, View v, int position, long id) {
    showTour((File) getListAdapter().getItem(position));
  }

  @Override
  protected void onResume() {
    super.onResume();
    TourModel.readTours();
    ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
    adapter.clear();
    for (File file : TourModel.tours) {
      adapter.add(file);
    }
    scroll(adapter.getPosition(TourModel.tour));
  }

  /** Remove tour from tours and from filesystem. */
  private void removeTour(File tour) {
    ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
    adapter.remove(tour);
    TourModel.removeTour(tour);
  }

  /** Rename tour. */
  public void renameTour(final File tour) {
    AlertDialog.Builder alert = new AlertDialog.Builder(this);
    alert.setIcon(R.drawable.folder);
    alert.setTitle(R.string.alert_title_tour_rename);
    LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    final EditText editText = (EditText) inflater.inflate(R.layout.tour_rename, null);
    final String nameOld = tour.getName().substring(0, tour.getName().lastIndexOf("."));
    editText.setText(nameOld);
    alert.setView(editText);
    alert.setPositiveButton(R.string.alert_ok_tour_rename, new DialogInterface.OnClickListener() {
      public void onClick(DialogInterface dialog, int whichButton) {
        String name = editText.getText().toString();
        if (0 == name.length() || name.equalsIgnoreCase(nameOld)) {
          return;
        }
        File tourNew = new File(tour.getParentFile(), name + TourModel.EXTENSION);
        if(tour.renameTo(tourNew)) {
          File tourPath = TourModel.getTourPath(tour);
          File tourPathNew = new File(tour.getParentFile(), name);
          tourPath.renameTo(tourPathNew);
          TourModel.tours.remove(tour);
          TourModel.tours.add(tourNew);
          TourModel.tour = tourNew;
          Collections.sort(TourModel.tours);
          try {
            // update name in gpx file
            TourModel.readTourpoints();
            TourModel.writeTourpoints();
          } catch (Exception ignore) {
          }
          ArrayAdapter<File> adapter = (ArrayAdapter<File>) getListAdapter();
          adapter.clear();
          for (File tour : TourModel.tours) {
            adapter.add(tour);
          }
          scroll(adapter.getPosition(TourModel.tour));
        }
      }
    });
    alert.show();
  }

  /** Show specified tour. */
  private void showTour(File tour) {
    TourModel.tour = tour;
    Intent intent = new Intent(this, TourpointsActivity.class);
    startActivity(intent);
  }
}