package com.lindosoft.android.guide.model;

import android.location.*;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/** Read and write gpx files. */
public class GpxModel extends DefaultHandler {
  public List<Point> trackpoints = new ArrayList<Point>();
  public List<Point> waypoints = new ArrayList<Point>();
  private double altitude = 0.0D;
  private StringBuilder builder = new StringBuilder();
  private String name = null;
  private Point point = null;

  /** Parse xml from *.gpx file. */
  public void readGpx(File in) throws Exception {
    SAXParserFactory parserFactory = SAXParserFactory.newInstance();
    SAXParser parser = parserFactory.newSAXParser();
    XMLReader reader = parser.getXMLReader();
    // does not work: XMLReader reader = XMLReaderFactory.createXMLReader();
    reader.setContentHandler(this);
    reader.parse(new InputSource(new FileInputStream(in)));
  }

  @Override
  public void characters(char[] ch, int start, int length) {
    builder.append(ch, start, length);
  }

  @Override
  public void startElement(String uri, String localName, String qName, Attributes attributes) {
    if ("ele".equals(localName) || "name".equals(localName)) {
      builder = new StringBuilder();
    }
    if ("rpt".equals(localName) || "rtept".equals(localName) || "trkpt".equals(localName) || "wpt".equals(localName)) {
      point = new Point();
      Location location = new Location(LocationManager.GPS_PROVIDER);
      location.setLatitude(Double.parseDouble(attributes.getValue("lat")));
      location.setLongitude(Double.parseDouble(attributes.getValue("lon")));
      point.location = location;
    }
  }

  @Override
  public void endElement(String uri, String localName, String qName) {
    if ("ele".equals(localName)) {
      altitude = Double.parseDouble(builder.toString());
    } else if ("name".equals(localName)) {
      name = builder.toString();
    } else if ("rpt".equals(localName) || "rtept".equals(localName) || "trkpt".equals(localName)) {
      point.location.setAltitude(altitude);
      point.position = trackpoints.size();
      trackpoints.add(point);
    } else if ("wpt".equals(localName)) {
      point.location.setAltitude(altitude);
      point.name = name;
      point.position = waypoints.size();
      waypoints.add(point);
    }
  }

  public void writeGpx(File out, List<Point> waypoints) throws Exception {
    PrintWriter printer = new PrintWriter(out, "UTF-8");
    try {
      writeGpxHead(out, printer);
      for (Point waypoint : waypoints) {
        writeGpxWaypoint(printer, waypoint);
      }
      writeGpxFoot(printer);
      printer.flush();
    } finally {
      printer.close();
      printer = null;
    }
  }

  /** Write gpx end element. */
  private void writeGpxFoot(PrintWriter printer) {
    printer.println("</gpx>");
  }

  /** Write gpx start element. */
  private void writeGpxHead(File out, PrintWriter printer) {
    printer.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    printer.println("<gpx xmlns=\"http://www.topografix.com/GPX/1/1\" creator=\"lindoGuide\" version=\"1.1\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd\">");
    printer.println("<metadata>");
    printer.println("  <name>" + out.getName() + "</name>");
    printer.println("  <time>" + new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").format(new Date()) + "</time>");
    printer.println("</metadata>");
  }

  /** Write wpt element. */
  private void writeGpxWaypoint(PrintWriter printer, Point waypoint) {
    printer.println("<wpt lat=\"" + waypoint.location.getLatitude() + "\" lon=\"" + waypoint.location.getLongitude() + "\">");
    printer.println("  <ele>" + waypoint.location.getAltitude() + "</ele>");
    printer.println("  <name>" + waypoint.name + "</name>");
    printer.println("  <sym>Waypoint</sym>");
    printer.println("  <type>Generic</type>");
    printer.println("</wpt>");
  }
}