package com.lindosoft.android.guide.model;

import android.location.Location;
import android.location.LocationManager;
import android.os.Environment;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

/** Shared data for sight activities. */
public abstract class SightModel {
  public static final String BASE = "guides";
  public static final String SEPARATOR = "_";
  public static File sight = null;
  public static List<File> sights = new ArrayList<File>();
  public static Point sightpoint = null;
  public static List<Point> sightpoints = new ArrayList<Point>();

  /** Create new sight. */
  public static File createSight(String name) {
    HashSet names = new HashSet();
    for (File sightTemp : sights) {
      names.add(sightTemp.getName());
    }
    if (0 == name.length() || names.contains(name)) {
      int i = 9999;
      String nameNext = "" + i;
      do {
        name = nameNext;
        i--;
        nameNext = "" + i;
        while (nameNext.length() < 4) {
          nameNext = "0" + nameNext;
        }
      } while (i > 0 && (names.contains(name) || !names.contains(nameNext)));
    }
    sight = new File(getBase(), name);
    sight.mkdirs();
    sights.add(sight);
    return sight;
  }

  /** Create new sightpoint. */
  public static Point createSightpoint(Location location, String name) {
    boolean headerPresent = false;
    for (Point sightpointTemp : sightpoints) {
      if (null == sightpointTemp.location) {
        headerPresent = true;
        break;
      }
    }
    if (headerPresent && null != location) {
      name = round(location.getLatitude()) + SEPARATOR + round(location.getLongitude()) + (0 == name.length() ? "" : SEPARATOR + name);
    } else {
      name = sight.getName();
    }
    name = name + MainModel.audioExtension;
    sightpoint = createSightpointFromName(name);
    return sightpoint;
  }

  /** Get absolute path of sightpoint file. */
  public static String getSightpointAbsolutePath() {
    return new File(sight, sightpoint.name).getAbsolutePath();
  }

  /** Read all sights from filesystem. */
  public static void readSights() {
    sightpoint = null;
    sightpoints.clear();
    sight = null;
    sights.clear();
    File[] children = getBase().listFiles();
    for (File child : children) {
      if (child.isDirectory()) {
        sights.add(child);
      }
    }
    Collections.sort(sights);
  }

  /** Read sightpoints from sight directory. */
  public static void readSightpoints() {
    sightpoints.clear();
    File[] children = sight.listFiles();
    for (File child : children) {
      sightpoints.add(createSightpointFromName(child.getName()));
    }
    Collections.sort(sightpoints);
  }

  /** Remove sight from sights and from filesystem. */
  public static void removeSight(File sightTemp) {
    sight = null;
    sights.remove(sightTemp);
    for (File file : sightTemp.listFiles()) {
      file.delete();
    }
    sightTemp.delete();
  }

  /** Remove sightpoint from sight and from filesystem. */
  public static void removeSightpoint(Point sightpointTemp) {
    sightpoint = null;
    sightpoints.remove(sightpointTemp);
    new File(sight, sightpointTemp.name).delete();
  }


  /** Create point from name. */
  private static Point createSightpointFromName(String name) {
    Point sightpoint = new Point();
    sightpoint.name = name;
    try {
      int index = name.lastIndexOf(".");
      String[] tokens = name.substring(0, index).split(SEPARATOR);
      Location location = new Location(LocationManager.GPS_PROVIDER);
      location.setAltitude(0.0d);
      location.setLatitude(Double.parseDouble(tokens[0]));
      location.setLongitude(Double.parseDouble(tokens[1]));
      sightpoint.location = location;
    } catch (Exception ex) {
    }
    return sightpoint;
  }

  /** Get base directory. */
  private static File getBase() {
    File home = new File(Environment.getExternalStorageDirectory(), MainModel.HOME);
    File base = new File(home, BASE);
    base.mkdirs();
    return base;
  }

  /** Limit/pad double to fixed number of digits. */
  private static String round(double in) {
    String result = in + "000000";
    return result.substring(0, result.indexOf(".") + 7); // point and 6 digits
  }
}