package com.lindosoft.android.guide.model;

import android.location.Location;
import android.os.Environment;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

/** Shared data for tour activities. */
public class TourModel {
  public static final String BASE = "tours";
  public static final String EXTENSION = ".gpx";
  public static final String SEPARATOR = "_";
  public static File tour = null;
  public static List<File> tours = new ArrayList<File>();
  public static Point tourpoint = null;
  public static List<Point> tourpoints = new ArrayList<Point>();

  /** Create new tour. */
  public static File createTour(String name) throws Exception {
    HashSet names = new HashSet();
    for (File tourTemp : tours) {
      names.add(tourTemp.getName());
    }
    if (0 == name.length() || names.contains(name)) {
      int i = 9999;
      String nameNext = "" + i;
      do {
        name = nameNext;
        i--;
        nameNext = "" + i;
        while (nameNext.length() < 4) {
          nameNext = "0" + nameNext;
        }
      } while (i > 0 && (names.contains(name) || !names.contains(nameNext)));
    }
    tour = new File(getBase(), name + EXTENSION);
    tourpoints.clear();
    tourpoint = null;
    GpxModel gpxModel = new GpxModel();
    gpxModel.writeGpx(tour, tourpoints);
    tours.add(tour);
    return tour;
  }

  /** Create new tourpoint. */
  public static Point createTourpoint(Location location, String name) throws Exception {
    Point tourpointNew = new Point();
    tourpointNew.name = name;
    tourpointNew.location = location;
    tourpoints.add(tourpointNew);
    writeTourpoints();
    tourpoint = tourpointNew;
    return tourpointNew;
  }
    
  /** Get direction, return null if not available. */
  public static int getDirection(Location location, Location locationNow, Location locationTarget) {
    if (location.hasBearing()) {
      int bearing = (int) location.getBearing();
      int bearingTarget = null == locationNow ? (int) location.bearingTo(locationTarget)
          : (int) locationNow.bearingTo(locationTarget);
      return (((bearingTarget - bearing + 360 + 345) / 30) % 12) + 1;
    }
    return 12;
  }

  /** Get distance. */
  public static int getDistance(Location location, Location locationTarget) {
    int distance = (int) location.distanceTo(locationTarget);
    if (distance < 100) {
      distance = (distance / 10) * 10;
    } else if (distance < 1000) {
      distance = (distance / 100) * 100;
    } else if (distance < 10000) {
      distance = (distance / 1000) * 1000;
    } else {
      distance = 10000;
    }
    return distance;
  }

  /** Get tour directory. */
  public static File getTourPath(File tourTemp) {
    File tourPath = new File(getBase(), tourTemp.getName().substring(0, tourTemp.getName().lastIndexOf(".")));
    tourPath.mkdirs();
    return tourPath;
  }

  /** Get absolute path of tourpoint file. */
  public static String getTourpointAbsolutePath() {
    String name = tourpoint.location.getLatitude() + SEPARATOR + tourpoint.location.getLongitude() + SEPARATOR + tourpoint.name;
    return new File(getTourPath(tour), name).getAbsolutePath();
  }

  /** Get position of tourpoint. */
  public static int getTourpointPosition() {
    return tourpoints.indexOf(tourpoint);
  }

  /** Read tourpoints of current tour from filesystem. */
  public static void readTourpoints() throws Exception {
    GpxModel gpxModel = new GpxModel();
    gpxModel.readGpx(tour);
    tourpoints.clear();
    tourpoints.addAll(gpxModel.waypoints);
    if (-1 == tourpoints.indexOf(tourpoint) && 0 != tourpoints.size()) {
      tourpoint = tourpoints.get(0);
    }
  }
      
  /** Read all tours from the filesystem. */
  public static void readTours() {
    List<File> toursTemp = new ArrayList<File>();
    File[] children = getBase().listFiles();
    for (File child : children) {
      if (child.getName().endsWith(EXTENSION)) {
        toursTemp.add(child);
      }
    }
    Collections.sort(toursTemp);
    tours.clear();
    tours.addAll(toursTemp);
  }

  /** Remove tour from tours and from filesystem. */
  public static void removeTour(File tourTemp) {
    tour = null;
    tours.remove(tourTemp);
    tourTemp.delete();
    File tourPath = getTourPath(tourTemp);
    for (File file : tourPath.listFiles()) {
      file.delete();
    }
    tourPath.delete();
  }

  /** Remove tourpoint from tourpoints and from filesystem. */
  public static void removeTourpoint(Point tourpointTemp) throws Exception {
    tourpoint = null;
    tourpoints.remove(tourpointTemp);
    writeTourpoints();
  }

  /** Set tourpoint nearest to location. */
  public static void setNearest(Location location) {
    if (null == location) {
      return;
    }
    float distanceNearest = location.distanceTo(tourpoints.get(0).location);
    Point tourpointNearest = tourpoints.get(0);
    for (int position=0; position<tourpoints.size(); position++) {
      float distance = location.distanceTo(tourpoints.get(position).location);
      if (distance < distanceNearest) {
        distanceNearest = distance;
        tourpointNearest = tourpoints.get(position);
      }
    }
    tourpoint = tourpointNearest;
  }

  /** Write tourpoints of current tour to filesystem. */
  public static void writeTourpoints() throws Exception {
    GpxModel gpxModel = new GpxModel();
    gpxModel.writeGpx(tour, tourpoints);
  }


  /** Get base directory. */
  private static File getBase() {
    File home = new File(Environment.getExternalStorageDirectory(), MainModel.HOME);
    File base = new File(home, BASE);
    base.mkdirs();
    return base;
  }
}